﻿using System.Windows;
using DomainModel;
using IdeaBlade.EntityModel;

namespace WpfApplication1 {

  // Sample Push client.
  // This sample will subscribe to two Push services:
  //   1) "LongRunningProcedure" - a service which sends period notifications to all its subscribers
  //   2) "OrderWatcher" - a service which sends notifications to subscribers based upon subscriber-specific arguments.


  public partial class MainWindow : Window {

    private NorthwindIBEntities _entityManager;
    private string _owToken;

    public MainWindow() {
      InitializeComponent();
      this.Loaded += new RoutedEventHandler(MainWindow_Loaded);
    }

    void MainWindow_Loaded(object sender, RoutedEventArgs e) {
      // When connecting to a distributed BOS, it's always a good idea
      // to set up an error handler and issue an explicit Connect.
      _entityManager = new NorthwindIBEntities(false);
      _entityManager.EntityServerError += (o, args) => {
        MessageBox.Show(args.Exception.Message);
      };
      _entityManager.Connect();
    }

    private void btnSubscribeLRP_Click(object sender, RoutedEventArgs e) {
      SubscribeLRP();
    }

    private void btnSubscribeOW_Click(object sender, RoutedEventArgs e) {
      SubscribeOW();
    }

    private void btnUnsubscribeOW_Click(object sender, RoutedEventArgs e) {
      UnsubscribeOW();
    }

    #region LongRunningProcedure
    // Subscribe to a LongRunningProcedure, which provides periodic status notifications.
    // When complete, we automatically unsubscribe from it.
    // Note that the RegisterCallback is executed asynchronously.  If it fails, the error
    // is passed to the callback.

    private void SubscribeLRP() {

      // Register for the service.
      _entityManager.RegisterCallback(
        "DomainModel.LongRunningProcedure, DomainModel",      /* Type name */
        "MultiStepService",                                   /* Method name */
        LongRunningProcedureNotification,                     /* the callback */
        "lrp");                                               /* user token */

      this.btnSubscribeLRP.IsEnabled = false;

      MessageBox.Show("You are now subscribed to the LongRunningProcedure.");
    }

    private void LongRunningProcedureNotification(SubscriptionOperation op) {
      if (op.HasError) {
        MessageBox.Show(op.Error.Message, "Subscription Error");
        this.btnSubscribeLRP.IsEnabled = false;
        return;
      }

      // This service returns one argument.
      LongRunningInfo info = op.NotificationData[0] as LongRunningInfo;
      string msg = info.Message;
      int workComplete = info.CompletionPercent;

      MessageBox.Show(msg, "Received from LongRunningProcedure");

      // We can unsubscribe once the procedure completes.
      if (workComplete == 100) {
        op.CancelCallback();
        this.btnSubscribeLRP.IsEnabled = true;
      }
    }
    #endregion


    #region OrderWatcher subscribe/unsubscribe

    // Remember to add orders (manually or through another program) to
    // have OrderWatcher broadcast messages.  This will do:
    //    insert into [Order](customerid, employeeid, orderdate, rowversion)
    //    select customerid, 1, getdate(), 1 from customer where CompanyName = 'QUICK-Stop'

    private void SubscribeOW() {
      // The server-side code to be run.
      ServerNotifyDelegate serverDel = DomainModel.OrderWatcher.NewOrderService;

      _owToken = "2";     // Arbitrary user token - we'll use this to unsubscribe.

      int employeeId = 1;  // We'll pass this as a "client argument" into the service.

      // Register
      _entityManager.RegisterCallback(
        serverDel,                // The service method
        OrderPushCallback,        // the callback action
        _owToken,                 // Token identifying this subscription
        employeeId);              // Client argument passed into the service

      // This example contains only a single subscription to this service from this client.  You
      // can subscribe to this or other services multiple times - each subscription
      // needs to use a unique UserToken.
      this.btnUnsubscribeOW.IsEnabled = true;
      this.btnSubscribeOW.IsEnabled = false;

      MessageBox.Show("You are now subscribed to the OrderWatcher.");
    }

    private void UnsubscribeOW() {
      // Cancel based on the UserToken we subscribed with.
      _entityManager.CancelCallback(_owToken);

      this.btnUnsubscribeOW.IsEnabled = false;
      this.btnSubscribeOW.IsEnabled = true;
      MessageBox.Show("You are now unsubscribed from the OrderWatcher.");
    }

    private void OrderPushCallback(SubscriptionOperation op) {
      // We know the service is returning a single message string...
      string msg = op.NotificationData[0].ToString();
      MessageBox.Show(msg, "Received from OrderWatcher");
    }
  
    #endregion
  }
}
